@@ -10,6 +10,7 @@ from importlib import metadata
 from typing import Any, TypedDict
 
 import async_timeout
+import backoff
 from aiohttp.client import ClientError, ClientResponseError, ClientSession
 from aiohttp.hdrs import METH_POST
 from aiohttp.helpers import BasicAuth
@@ -18,16 +19,21 @@ from yarl import URL
 
 from .constants import (
     DEVICE_INFO_API_V1,
-    DEVICE_INFO_API_V2,
+    DEVICE_INFO_API_V3,
     HEATING_CIRCUIT1_API_V1,
-    HEATING_CIRCUIT1_API_V2,
+    HEATING_CIRCUIT1_API_V3,
     HVAC_MODE_DICT,
     HVAC_MODE_DICT_REVERSE,
+    INVALID_VALUES_ERROR_MSG,
+    NO_STATE_ERROR_MSG,
     SENSORS_API_V1,
-    SENSORS_API_V2,
+    SENSORS_API_V3,
+    STATIC_VALUES_API_V1,
+    STATIC_VALUES_API_V3,
+    VERSION_ERROR_MSG,
 )
 from .exceptions import BSBLANConnectionError, BSBLANError
-from .models import Device, Info, Sensor, State
+from .models import Device, Info, Sensor, State, StaticState
 
 logging.basicConfig(level=logging.DEBUG)
 
@@ -41,31 +47,37 @@ class BSBLAN:
     password: str | None = None
     passkey: str | None = None
     port: int = 80
-    request_timeout: int = 8
+    request_timeout: int = 10
     session: ClientSession | None = None
     _version: str = ""
-    _heating_circuit1: str | None = None
+    _heating_params: list[str] | None = None
+    _string_circuit1: str | None = None
     _sensor_params: list[str] | None = None
     _sensor_list: str | None = None
-    _heating_params: list[str] | None = None
+    _static_params: list[str] | None = None
+    _static_list: str | None = None
+    _device_params: list[str] = field(default_factory=list)
+    _min_temp: float = 7.0
+    _max_temp: float = 25.0
     _info: str | None = None
-    _device_params: list = field(default_factory=list)
     _auth: BasicAuth | None = None
     _close_session: bool = False
 
+    @backoff.on_exception(backoff.expo, BSBLANConnectionError, max_tries=3, logger=None)
     async def _request(
         self,
         method: str = METH_POST,
         base_path: str = "/JQ",
-        data: dict | None = None,
-        params: dict | None = None,
-    ) -> dict[str, Any]:
+        data: dict[str, object] | None = None,
+        params: dict[str, object] | None = None,
+    ) -> Any:
         """Handle a request to a BSBLAN device.
 
         A generic method for sending/handling HTTP requests done against
         the BSBLAN API.
 
         Args:
+        ----
             method: HTTP method to use.
             base_path: Base path to use.
             data: Dictionary of data to send to the BSBLAN device.
@@ -73,16 +85,17 @@ class BSBLAN:
                 retrieve certain data.
 
         Returns:
+        -------
             A Python dictionary (Json decoded) with the response from
             the BSBLAN API.
 
         Raises:
+        ------
             BSBLANConnectionError: If the connection to the BSBLAN device
                 fails.
             BSBLANError: If receiving from the BSBLAN device an unexpected
                 response.
         """
-
         try:
             version = metadata.version(__package__ or __name__)
         except metadata.PackageNotFoundError:
@@ -93,7 +106,10 @@ class BSBLAN:
             base_path = f"/{self.passkey}{base_path}"
 
         url = URL.build(
-            scheme="http", host=self.host, port=self.port, path=base_path
+            scheme="http",
+            host=self.host,
+            port=self.port,
+            path=base_path,
         ).join(URL())
 
         if self._auth is None and self.username and self.password:
@@ -120,107 +136,87 @@ class BSBLAN:
                 )
                 response.raise_for_status()
         except asyncio.TimeoutError as exception:
-            raise BSBLANConnectionError(
-                "Timeout occurred while connecting to BSBLAN device."
-            ) from exception
+            raise BSBLANConnectionError(BSBLANConnectionError.message) from exception
         except (
             ClientError,
             ClientResponseError,
             socket.gaierror,
         ) as exception:
-            raise BSBLANConnectionError(
-                "Error occurred while communicating with BSBLAN device."
-            ) from exception
+            raise BSBLANConnectionError(BSBLANConnectionError.message) from exception
 
         content_type = response.headers.get("Content-Type", "")
         if "application/json" not in content_type:
             text = await response.text()
-            raise BSBLANError(
-                "Unexpected response from the BSBLAN device",
-                {"Content-Type": content_type, "response": text},
-            )
+            raise BSBLANError(BSBLANError.message + f" ({text})")
 
         return await response.json()
 
-    async def _scan(self, parameters: list) -> str:
-        """Scan for parameters that return a value.
-
-        Args:
-            parameters: List of parameters to scan.
-
-        Returns:
-            String of valid parameters.
-
-        """
-
-        # convert list to string
-        _parameters = ",".join(map(str, parameters))
-        data = await self._request(params={"Parameter": f"{_parameters}"})
-        not_valid_data = []
-        not_valid_data2 = []
-        for key, value in data.items():
-            if not value.get("value"):
-                not_valid_data.append(key)
-            if value.get("value") == "":
-                not_valid_data2.append(key)
-
-        # remove parameters with no returning value
-        for i in not_valid_data or not_valid_data2:
-            data.pop(i)
-
-        # join parameters to create one string
-        parameters = []
-        for i in data.keys():
-            parameters.append(i)
-        _parameters = ",".join(map(str, parameters))
-
-        return _parameters
-
     async def state(self) -> State:
         """Get the current state from BSBLAN device.
 
-        Returns:
+        Returns
+        -------
             A BSBLAN state object.
         """
-        if not self._heating_circuit1 or not self._heating_params:
+        if not self._string_circuit1 or not self._heating_params:
+            # retrieve heating circuit 1
             data = await self._get_dict_version()
-            logger.debug("data: %s", data)
             data = await self._get_parameters(data["heating"])
-            self._heating_circuit1 = str(data["string_par"])
+            self._string_circuit1 = str(data["string_par"])
             self._heating_params = list(data["list"])
 
         # retrieve heating circuit 1 and heating params so we can build the
         # data structure (its circuit 1 because it can support 2 circuits)
-        logger.debug("get state")
-        data = await self._request(params={"Parameter": f"{self._heating_circuit1}"})
-        data = dict(zip(self._heating_params, list(data.values())))
+        data = await self._request(params={"Parameter": f"{self._string_circuit1}"})
+        data = dict(zip(self._heating_params, list(data.values()), strict=True))
+
+        # set hvac_mode with correct value
         data["hvac_mode"]["value"] = HVAC_MODE_DICT[int(data["hvac_mode"]["value"])]
-        return State.parse_obj(data)
+        return State.model_validate(data)
 
     async def sensor(self) -> Sensor:
         """Get the sensor information from BSBLAN device.
 
-        Returns:
+        Returns
+        -------
             A BSBLAN sensor object.
         """
         if not self._sensor_params:
             data = await self._get_dict_version()
-            logger.debug("data: %s", data)
             data = await self._get_parameters(data["sensor"])
-            logger.debug("data: %s", data)
             self._sensor_list = str(data["string_par"])
             self._sensor_params = list(data["list"])
 
         # retrieve sensor params so we can build the data structure
-        logger.debug("get sensor data")
         data = await self._request(params={"Parameter": f"{self._sensor_list}"})
-        data = dict(zip(self._sensor_params, list(data.values())))
-        return Sensor.parse_obj(data)
+        data = dict(zip(self._sensor_params, list(data.values()), strict=True))
+        return Sensor.model_validate(data)
 
-    async def _get_dict_version(self) -> dict:
+    async def static_values(self) -> StaticState:
+        """Get the static information from BSBLAN device.
+
+        Returns
+        -------
+            A BSBLAN staticState object.
+        """
+        if not self._static_params:
+            data = await self._get_dict_version()
+            data = await self._get_parameters(data["staticValues"])
+            self._static_list = str(data["string_par"])
+            self._static_params = list(data["list"])
+
+        # retrieve sensor params so we can build the data structure
+        data = await self._request(params={"Parameter": f"{self._static_list}"})
+        data = dict(zip(self._static_params, list(data.values()), strict=True))
+        self._min_temp = data["min_temp"]["value"]
+        self._max_temp = data["max_temp"]["value"]
+        return StaticState.model_validate(data)
+
+    async def _get_dict_version(self) -> dict[Any, Any]:
         """Get the version from device.
 
-        Returns:
+        Returns
+        -------
             A dictionary with dicts
 
         """
@@ -231,35 +227,35 @@ class BSBLAN:
         if pkg_version.parse(self._version) < pkg_version.parse("1.2.0"):
             return {
                 "heating": HEATING_CIRCUIT1_API_V1,
+                "staticValues": STATIC_VALUES_API_V1,
                 "device": DEVICE_INFO_API_V1,
                 "sensor": SENSORS_API_V1,
             }
-        if (
-            pkg_version.parse("3.0.0")
-            > pkg_version.parse(self._version)  # noqa: W503
-            >= pkg_version.parse("2.0.0")  # noqa: W503
-        ):
+        if pkg_version.parse(self._version) > pkg_version.parse("3.0.0"):
             return {
-                "heating": HEATING_CIRCUIT1_API_V2,
-                "device": DEVICE_INFO_API_V2,
-                "sensor": SENSORS_API_V2,
+                "heating": HEATING_CIRCUIT1_API_V3,
+                "staticValues": STATIC_VALUES_API_V3,
+                "device": DEVICE_INFO_API_V3,
+                "sensor": SENSORS_API_V3,
             }
-        return {}
+        raise BSBLANError(VERSION_ERROR_MSG)
 
     async def device(self) -> Device:
         """Get BSBLAN device info.
 
-        Returns:
+        Returns
+        -------
             A BSBLAN device info object.
 
         """
         device_info = await self._request(base_path="/JI")
-        return Device.parse_obj(device_info)
+        return Device.model_validate(device_info)
 
     async def info(self) -> Info:
         """Get information about the current heating system config.
 
-        Returns:
+        Returns
+        -------
             A BSBLAN info object about the heating system.
         """
         if not self._info or not self._device_params:
@@ -269,24 +265,26 @@ class BSBLAN:
             self._device_params = data["list"]
 
         data = await self._request(params={"Parameter": f"{self._info}"})
-        data = dict(zip(self._device_params, list(data.values())))
-        return Info.parse_obj(data)
+        data = dict(zip(self._device_params, list(data.values()), strict=True))
+        return Info.model_validate(data)
 
-    async def _get_parameters(self, params: dict) -> dict:
+    async def _get_parameters(self, params: dict[Any, Any]) -> dict[Any, Any]:
         """Get the parameters info from BSBLAN device.
 
         Args:
+        ----
             params: A dictionary with the parameters to get.
 
         Returns:
-            A list of 2 objects [str, list].
+        -------
+            A dict of 2 objects [str, list].
         """
-        list_params = [*params]
-        parameters = await self._scan(list_params)
-        logger.debug("parameters from scan: %s", parameters)
-        object_parameters = list(params.values())
+        _string_params = [*params]
+        list_params = list(params.values())
+        # convert list of string to string
+        string_params = ",".join(map(str, _string_params))
 
-        return {"string_par": parameters, "list": object_parameters}
+        return {"string_par": string_params, "list": list_params}
 
     async def thermostat(
         self,
@@ -296,15 +294,18 @@ class BSBLAN:
         """Change the state of the thermostat through BSB-Lan.
 
         Args:
+        ----
             target_temperature: Target temperature to set.
             hvac_mode: Preset mode to set.
 
         Raises:
+        ------
             BSBLANError: The provided values are invalid.
         """
 
         class ThermostatState(  # lgtm [py/unused-local-variable]
-            TypedDict, total=False
+            TypedDict,
+            total=False,
         ):
             """Describe state dictionary that can be set on the thermostat."""
 
@@ -318,28 +319,30 @@ class BSBLAN:
         state: ThermostatState = {}
 
         if target_temperature is not None:
-            if not 7 <= float(target_temperature) <= 40:
-                raise BSBLANError(
-                    "Target temperature is not valid, must be between 7 and 40"
-                )
+            if not (
+                float(self._min_temp)
+                <= float(target_temperature)
+                <= float(self._max_temp)
+            ):
+                raise BSBLANError(INVALID_VALUES_ERROR_MSG)
             state["Parameter"] = "710"
             state["Value"] = target_temperature
             state["Type"] = "1"
 
         if hvac_mode is not None:
             if hvac_mode not in HVAC_MODE_DICT_REVERSE:
-                raise BSBLANError("HVAC mode is not valid")
+                raise BSBLANError(INVALID_VALUES_ERROR_MSG)
             state["Parameter"] = "700"
             state["EnumValue"] = HVAC_MODE_DICT_REVERSE[hvac_mode]
             state["Type"] = "1"
 
         if not state:
-            raise BSBLANError("No state provided")
+            raise BSBLANError(NO_STATE_ERROR_MSG)
 
         # Type needs to be 1 to really set value.
         # Now it only checks if it could set value.
         response = await self._request(base_path="/JS", data=dict(state))
-        logger.debug("response: %s", response)
+        logger.debug("response for setting: %s", response)
 
     async def close(self) -> None:
         """Close open client session."""
@@ -349,16 +352,17 @@ class BSBLAN:
     async def __aenter__(self) -> BSBLAN:
         """Async enter.
 
-        Returns:
+        Returns
+        -------
             The BSBLAN object.
         """
-
         return self
 
-    async def __aexit__(self, *_exc_info: Any) -> None:
+    async def __aexit__(self, *_exc_info: object) -> None:
         """Async exit.
 
         Args:
+        ----
             *_exc_info: Exec type.
         """
         await self.close()
